; Trick12.asm
; This demo displays 12 sprites/row with two different colors
; Copyright 2001 Thomas Jentzsch

    processor 6502
    include vcs.h

;===============================================================================
; C O N S T A N T S
;===============================================================================

OFF_COLOR       = $46   ; red
ON_COLOR        = $ca   ; green
OFF_COLOR1      = $88   ; blue
ON_COLOR1       = $2e   ; orange

SPRITE_HEIGHT   = 7     ; head
SPRITE_HEIGHT1  = 9     ; body

;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

tmpVar      .byte
l01         .byte
l2          .byte
r0          .byte
r1          .byte
l01_u       .byte
l2_u        .byte
r0_u        .byte
r1_u        .byte
FrameCnt    .byte


;===============================================================================
; R O M - C O D E
;===============================================================================
    SEG     Bank0
    ORG     $f000

;***********************************************************
Start SUBROUTINE
;***********************************************************
    sei                         ;           disable interrupts, if there are any.
    cld                         ;           clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop

    jsr     GameInit

;schema copied from "How to draw a playfield" :)
.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop

;***********************************************************
GameInit SUBROUTINE
;***********************************************************
; init the playfield (color) masks:
    lda     #%00001100
    sta     l01
    lda     #%11110011
    sta     l2
    lda     #%00110000
    sta     r0
    lda     #%00001100
    sta     r1
    lda     #%00001111
    sta     l01_u
    lda     #%11111100
    sta     l2_u
    lda     #%00001100
    sta     r0_u
    lda     #%11001100
    sta     r1_u
    rts

;***********************************************************
VerticalBlank SUBROUTINE
;***********************************************************
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr                 ;       -> bit 1 = 0 (= disable VSYNC)
    ldx     #44
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts

;***********************************************************
GameCalc SUBROUTINE
;***********************************************************
    inc     FrameCnt
    lda     FrameCnt
    and     #$1f        ;       change pattern every 32 frames (~0.5 sec)
    bne     .skip

; change the upper color pattern:
    lda     l2
    eor     #%11
    cmp     #$80
    rol
    cmp     #$80
    rol
    sta     l2
    eor     #%11001100
    sta     r1
    eor     #%11110000
    sta     r0
    eor     #%11110000
    and     #%1111
    sta     l01
; change the lower color pattern:
    lda     l2_u
    eor     #%0011
    cmp     #$80
    rol
    cmp     #$80
    rol
    sta     l2_u
    eor     #%11000011
    sta     r1_u
    eor     #%11110000
    sta     r0_u
    eor     #%11110000
    and     #%1111
    sta     l01_u

.skip:
    rts

; to avoid page crossing in branching:
    align 256

;***********************************************************
DrawScreen SUBROUTINE
;***********************************************************
    tsx
    stx     tmpVar

    ldx     #228            ;       228*64/76 = ~192 (=displayed lines)
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
    sta     VBLANK
    stx     TIM64T

    ldx     #$00            ;       normally the sprites/missiles
    ldy     #$00            ;        are black
    bit     INPT4
    bmi     .noFire
    ldx     #$0e            ;       if FIRE is pressed,
    ldy     #$1e            ;        change colors to show effect
.noFire:
    stx     COLUP0
    sty     COLUP1

    sta     WSYNC

    lda     #0
    sta     PF0
    sta     PF1
    sta     PF2

    lda     #ON_COLOR
    sta     COLUPF          ;       1st color comes from playfield
    lda     #0              ;       non-refelected playfield
    sta     CTRLPF
    lda     #%10011         ;       double width missiles and 3 player copies
    sta     NUSIZ0
    sta     NUSIZ1
    sta     RESM0           ; @32   position both missiles
    lda     #$90            ;        they are used to mask the holes
    sta     HMM0            ;        in the sprite pattern
    lda     #$80
    sta     HMM1
    ldx     r1
    sta     RESM1           ; @48
    txs                     ;       stack-pointer is used to save 1 cycle in loop

    sta     WSYNC
    sta     HMOVE

    ldx     #%10            ;
    stx     ENAM0
    stx     ENAM1

    ldx     #8
.wait:
    dex
    bne     .wait

; preload some values and enter the loop:
    ldy     #SPRITE_HEIGHT-1
    lda     SpriteTab,y
    sec

    ldx     l01
    stx     PF0
    stx     PF1

    ldx     #OFF_COLOR
    sta     WSYNC
    bit     $0
    bne     .enterLoop

; --------------------- PF Timings --------------------
; | PF0 |   PF1   |   PF2   | PF0 |   PF1   |   PF2   |
; |22.??|27 ..  ??|38 ..  ??|48.51|54 ..  ??|64 ..  ??|

; here starts the tricky code:
.loopPat:
    lda     l01             ; 3         the bits 4..7 are always 0, so the value can be used for PF0+1
    sta     PF0             ; 3 @75     left 0 (> @51 && <= @22, ok)
    sta     PF1             ; 3 @02     left 1 (<= 27, ok)
    lda     SpriteTab,y     ; 4
.enterLoop:
    sta     GRP0            ; 3 @09
    rol                     ; 2         shift pattern to display players with (nearly) the same distance (CF always set)
    sta     GRP1            ; 3 @14

    lda     l2              ; 3
    sta     PF2             ; 3 @20     left 2 (<= @38, ok)
    lda     r0              ; 3
    sta     RESP0           ; 3|26      the 1st 4 sprites
    sta     RESP1           ; 3|29
;two cycles have to be wasted here...
    stx.w   COLUBK          ; 4 @33     use background as 2nd color (*exact* timing essential here!)
    sta.w   RESP0           ; 4|37      the 2nd 4 sprites
    sta     RESP1           ; 3|40
    sta     PF0             ; 3 @43     right 0 (<= @48, ok)
    tsx                     ; 2         x = r1
    sta     RESP0           ; 3|48      the 3rd 4 sprites
    sta     RESP1           ; 3|51
    stx     PF1             ; 3 @54     right 1 (<= @54, ok) that was close :)
    lda     #0              ; 2
    sta     PF2             ; 3 @59     right 2 (<= @64, ok)
    ldx     #OFF_COLOR      ; 2
    dey                     ; 2
    sta     COLUBK          ; 3 @66     reset background to black (*exact* timing essential here!)
    bpl     .loopPat        ; 2

    sta     PF0             ;           disable playfield
    sta     PF1

; wait and do the same stuff again for the lower part:
    ldx     #8
.wait1:
    dex
    bne     .wait1

    ldx     r1_u
    txs
    lda     #ON_COLOR1
    sta     COLUPF

    ldy     #SPRITE_HEIGHT1-1; 2
    lda     SpriteTab1,y    ; 4
    ldx     l01_u           ; 3
    stx     PF0             ; 3
    stx     PF1             ; 3
    ldx     #OFF_COLOR1     ; 2
    sta     WSYNC
    bit     $0
    jmp     .enterLoop1

.loopPat1:
    lda     l01_u           ; 3
    sta     PF0             ; 3 @75     left 0
    sta     PF1             ; 3 @02     left 1
    lda     SpriteTab1,y     ; 4
.enterLoop1:
    sta     GRP0            ; 3 @09
    rol                     ; 2
    sta     GRP1            ; 3 @14

    lda     l2_u            ; 3
    sta     PF2             ; 3 @20     left 2
    lda     r0_u            ; 3
    sta     RESP0           ; 3|26
    sta     RESP1           ; 3|29
    stx.w   COLUBK          ; 4 @33
    sta.w   RESP0           ; 4|37
    sta     RESP1           ; 3|40
    sta     PF0             ; 3 @43     right 0
    tsx                     ; 2
    sta     RESP0           ; 3|48
    sta     RESP1           ; 3|51
    stx     PF1             ; 3 @54     right 1
    lda     #0              ; 2
    sta     PF2             ; 3 @59     right 2
    ldx     #OFF_COLOR1     ; 2
    dey                     ; 2
    sta     COLUBK          ; 3 @66
    bpl     .loopPat1       ; 2

    iny
    sty     PF0
    sty     PF1
    sty     GRP0
    sty     GRP1

    ldx     tmpVar
    txs

    sta     WSYNC
    sty     ENAM0           ; 3         disable one line later to show effect
    sty     ENAM1

    ldx     #2              ;           this demo is completely controled by the timers
.waitScreen:
    lda     INTIM
    bne     .waitScreen
    sta     WSYNC
    stx     VBLANK
    rts

;***********************************************************
OverScan SUBROUTINE
;***********************************************************
    lda     #35
    sta     TIM64T

; nothing to do...

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     $f700

; inverted pattern to mask the colored background/playfield:
SpriteTab:
    .byte   %11100011
    .byte   %11000001
    .byte   %11001001
    .byte   %11000001
    .byte   %11010101
    .byte   %11000001
    .byte   %11100011

SpriteTab1:
    .byte   %11000001
    .byte   %11000001
    .byte   %11000001
    .byte   %11000001
    .byte   %11100011
    .byte   %11100011
    .byte   %11100011
    .byte   %11100011
    .byte   %11100011

    org $f7fc
    .word   Start
    .word   Start
